"""
Train a classifier with a sample set.

@author Norman MacDonald
@date 2010-02-16
"""
import os,sys
from optparse import OptionParser, OptionGroup
from pica.Sample import SampleSet, ClassLabelSet
from pica.io.FileIO import FileIO
from pica.AssociationRule import load_rules
from pica.utils.ProgramTimer import ProgramTimer



if __name__ == "__main__":
	pt = ProgramTimer()
	parser = OptionParser(version="PICA %prog 1.0.1")
	parser.add_option("-a","--algorithm",action="store",dest="algorithm",help="Training algorithm [default = %default]",metavar="ALG",default="cpar.CPARTrainer")
	parser.add_option("-s","--samples",action="store",dest="input_samples_filename",help="Read samples from FILE",metavar="FILE")
	parser.add_option("-c","--classes",action="store",dest="input_classes_filename",help="Read class labels from FILE",metavar="FILE")
	parser.add_option("-t","--targetclass",action="store",dest="target_class",help="Set the target CLASS for testing",metavar="CLASS")
	parser.add_option("-o","--output",action="store",dest="output_filename",help="Write rules to FILE",metavar="FILE",default=None)
	parser.add_option("-p","--parameters",action="store",dest="parameters",help="FILE with additional, classifier-specific parameters. (confounders for CWMI)",metavar="FILE",default="taxonomic_confounders.txt")
	parser.add_option("-f","--feature_select",help="Model file (currently only association rule files) with features to select from [default: %default]",metavar="FILE",default=None)
	parser.add_option("-1","--feature_select_score",help="Order features by (feature selection option)", default="order_cwmi")
	parser.add_option("-n","--feature_select_top_n",help="Take the top n features(feature selection option)", type="int", default=20)

	(options, args) = parser.parse_args()

	
	pt.start()
	fileio = FileIO()
	samples = fileio.load_samples(options.input_samples_filename)
	samples_time = pt.stop()
	print "Loaded samples (%0.2fs)"%(samples_time)
	if options.feature_select:
		print "Selecting top %d features from %s, ordered by %s"%(options.feature_select_top_n,options.feature_select,options.feature_select_score)
		pt.start()
		from pica.AssociationRule import load_rules,AssociationRuleSet
		selected_rules = AssociationRuleSet()
		rules = load_rules(options.feature_select)
		rules.set_target_accuracy(options.feature_select_score)
		selected_rules.extend(rules[:options.feature_select_top_n])
		samples = samples.feature_select(selected_rules)
		print "Finished feature selection (%0.2fs)"%(pt.stop())
	classes = fileio.load_classes(options.input_classes_filename)
	samples.load_class_labels(classes)
	print samples.get_number_of_features()
	samples.set_current_class(options.target_class)
	
	pt.start()
	print "Compressing features...",
	samples = samples.compress_features()
	compression_time = pt.stop()
	print "\bfinished compression.(%0.2fs)"%(compression_time)
	samples.set_current_class(options.target_class)
	
	
	samples.hide_nulls(options.target_class)
	
	modulepath = "pica.trainers.%s"%(options.algorithm)
	classname = options.algorithm.split(".")[-1]
	TrainerClass = __import__(modulepath, fromlist=(classname,))
	trainer = TrainerClass.__dict__[classname](options.parameters)
	
	trainer.set_null_flag("NULL")
	pt.start()
	print "Starting training algorithm"
	rules = trainer.train(samples)
	training_time = pt.stop()
	if options.output_filename == None:
		options.output_filename = "%s.rules"%(options.target_class)
	rules.write(filename=options.output_filename)
	
	print "Finished training. (%0.2fs)"%(training_time)
	total_time = pt.end()
	print "Model output to %s. Total time: (%0.2fs)"%(options.output_filename,total_time)
	
	
